SystemOrganization addCategory: #'Seaside-Slime'!

RBParseTreeLintRule subclass: #WASlimeParseTreeLintRule
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Slime'!

WASlimeParseTreeLintRule subclass: #WAExtractCallbackCodeToMethodRule
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Slime'!

!WAExtractCallbackCodeToMethodRule methodsFor: 'accessing' stamp: 'lr 2/24/2009 15:07'!
group
	^ 'Seaside'! !

!WAExtractCallbackCodeToMethodRule methodsFor: 'initialization' stamp: 'lr 2/28/2009 22:57'!
initialize
	super initialize.
	self matcher
		matchesAnyOf: (WASlime callbackSelectors
			collect: [ :each | '`html `msg' , (self genericPatternForSelector: each) ])
		do: [ :node :answer |
			(answer isNil and: [ WASlime isBrushSelector: node receiver selector ]) 
				ifTrue: [
					node arguments
						detect: [ :each | each isBlock and: [ each body statements size > 1 ] ]
						ifNone: [ answer ] ]
				ifFalse: [ answer ] ]! !

!WAExtractCallbackCodeToMethodRule methodsFor: 'accessing' stamp: 'lr 2/24/2009 15:07'!
name
	^ 'Extract callback code to separate method'! !

!WAExtractCallbackCodeToMethodRule methodsFor: 'accessing' stamp: 'lr 2/24/2009 15:07'!
rationale
	^ 'For clarity rendering code and callback code should not be mixed, extract the contents of the callback block to a separate method.'! !

WASlimeParseTreeLintRule subclass: #WAFixCallbackTempsMissingRule
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Slime'!

!WAFixCallbackTempsMissingRule class methodsFor: 'testing' stamp: 'lr 8/18/2009 22:38'!
isVisible
	"Hide the test in closure images."

	^ [ ] isKindOf: BlockContext! !

!WAFixCallbackTempsMissingRule methodsFor: 'accessing' stamp: 'lr 2/24/2009 15:07'!
group
	^ 'Seaside'! !

!WAFixCallbackTempsMissingRule methodsFor: 'initialization' stamp: 'lr 12/3/2009 10:47'!
initialize
	super initialize.
	matcher := RBParseTreeSearcher new.
	matcher 
		matches: '[ | `@temps | `@.statements ]'
		do: [ :node :answer | 
			| found |
			found := false.
			(answer isNil and: [ node parent isMessage ]) ifTrue: [
				((self selectors includes: node parent selector) 
					or: [ (WASlime brushSelectors includes: node parent selector)
					or: [ (WASlime attributeSelectors includes: node parent selector)
					or: [ (WASlime callbackSelectors includes: node parent selector) ] ] ]) ifFalse: [
						| dangerousNames |
						dangerousNames := Set new.
						node parent parents do: [ :parent | 
							parent isBlock ifTrue: [ dangerousNames addAll: parent argumentNames ].
							parent isSequence ifTrue: [ dangerousNames addAll: parent temporaryNames ] ].
							(dangerousNames anySatisfy: [ :each | node references: each ]) 
								ifTrue: [ found := true ] ] ].
			found ifTrue: [ node ] ifFalse: [ answer ] ]! !

!WAFixCallbackTempsMissingRule methodsFor: 'accessing' stamp: 'lr 2/24/2009 15:07'!
name
	^ '#fixCallbackTemps possibly missing'! !

!WAFixCallbackTempsMissingRule methodsFor: 'accessing' stamp: 'lr 2/24/2009 15:07'!
rationale
	^ 'I don''t feel like explaining that here.'! !

!WAFixCallbackTempsMissingRule methodsFor: 'private' stamp: 'lr 2/27/2009 21:57'!
selectors
	^ #( allSatisfy: anySatisfy: at:ifAbsent: at:ifAbsentPut: at:ifPresent: count: critical: detect:ifNone: do: do:separatedBy: ensure: fixCallbackTemps fixTemps ifCurtailed: inject:into: noneSatisfy: on:do: reject: render: select: should: should:description: should:raise: should:raise:description: should:raise:whoseDescriptionDoesNotInclude:description: should:raise:whoseDescriptionIncludes:description: shouldnt: shouldnt:description: shouldnt:raise: shouldnt:raise:description: shouldnt:raise:whoseDescriptionDoesNotInclude:description: shouldnt:raise:whoseDescriptionIncludes:description: timesRepeat: use:during: with: )! !

WASlimeParseTreeLintRule subclass: #WAObjectInRule
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Slime'!

!WAObjectInRule commentStamp: 'pmm 9/12/2009 10:37' prior: 0!
I detect senders of Object >> #in:
!

!WAObjectInRule methodsFor: 'accessing' stamp: 'pmm 9/12/2009 10:36'!
group
	^ 'Seaside Portability'! !

!WAObjectInRule methodsFor: 'initialization' stamp: 'lr 11/2/2009 20:09'!
initialize
	super initialize.
	matcher := RBParseTreeSearcher new.
	matcher 
		matches: '``@object in: [ :`var | | `@temps | `@.statements ]'
		do: [ :node :answer | answer isNil ifTrue: [ node ] ifFalse: [ answer ] ]! !

!WAObjectInRule methodsFor: 'accessing' stamp: 'lr 9/12/2009 11:09'!
name
	^ 'Object>>#in:'! !

!WAObjectInRule methodsFor: 'accessing' stamp: 'lr 9/12/2009 11:09'!
rationale
	^ 'Object>>#in: has different semantics on different Smalltalk dialects.'! !

WASlimeParseTreeLintRule subclass: #WASendsRenderContentOnRule
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Slime'!

!WASendsRenderContentOnRule methodsFor: 'accessing' stamp: 'lr 2/24/2009 15:07'!
group
	^ 'Seaside'! !

!WASendsRenderContentOnRule methodsFor: 'initialization' stamp: 'lr 2/28/2009 21:10'!
initialize
	super initialize.
	self matcher
		matches: '`@object renderContentOn: `@html'
		do: [ :node :answer |
			(answer isNil
				and: [ (node methodNode selector ~= #visitPainter:)
				and: [ (node receiver isVariable and: [ node receiver name = 'super' ]) not ] ])
					ifTrue: [ node ]
					ifFalse: [ answer ] ]! !

!WASendsRenderContentOnRule methodsFor: 'accessing' stamp: 'lr 2/24/2009 15:07'!
name
	^ 'Sends #renderContentOn:'! !

!WASendsRenderContentOnRule methodsFor: 'accessing' stamp: 'lr 2/24/2009 15:07'!
rationale
	^ 'Client code should never send #renderContentOn: directly but only #render:'! !

!WASlimeParseTreeLintRule class methodsFor: 'testing' stamp: 'lr 2/24/2009 17:10'!
isVisible
	^ self name ~= #WASlimeParseTreeLintRule! !

WASlimeParseTreeLintRule subclass: #WAUnnecessaryBlockPassedToBrushRule
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Slime'!

!WAUnnecessaryBlockPassedToBrushRule methodsFor: 'running' stamp: 'lr 2/27/2009 22:13'!
checkMethod: aContext
	aContext isRenderingMethod 
		ifTrue: [ super checkMethod: aContext ]! !

!WAUnnecessaryBlockPassedToBrushRule methodsFor: 'accessing' stamp: 'lr 2/24/2009 15:07'!
group
	^ 'Seaside'! !

!WAUnnecessaryBlockPassedToBrushRule methodsFor: 'initialization' stamp: 'lr 2/28/2009 21:09'!
initialize
	super initialize.
	self matcher 
		matchesAnyOf: #(
			'`html `msg: [ ]'
			'`html `msg: [ `html text: ``@arg ]'
			'`html `msg: [ `html render: ``@arg ]' )
		do: [ :node :answer | 
			(answer isNil and: [ WASlime isBrushSelector: node selector ])
				ifTrue: [ node arguments first ]
				ifFalse: [ answer ] ].
	self matcher 
		matchesAnyOf: #(
			'`html `msg with: [ ]'
			'`html `msg with: [ `html text: ``@arg ]'
			'`html `msg with: [ `html render: ``@arg ]' )
		do: [ :node :answer | 
			(answer isNil and: [ WASlime isBrushSelector: node receiver selector ])
				ifTrue: [ node arguments first ]
				ifFalse: [ answer ] ]! !

!WAUnnecessaryBlockPassedToBrushRule methodsFor: 'accessing' stamp: 'lr 2/24/2009 15:07'!
name
	^ 'Unnecessary block passed to brush'! !

!WAUnnecessaryBlockPassedToBrushRule methodsFor: 'accessing' stamp: 'lr 2/24/2009 15:07'!
rationale
	^ 'Sending a block as argument to #with: is only needed when nesting brushes.'! !

WASlimeParseTreeLintRule subclass: #WAUnnecessaryWithSentToBrushRule
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Slime'!

!WAUnnecessaryWithSentToBrushRule methodsFor: 'running' stamp: 'lr 2/27/2009 22:38'!
checkMethod: aContext
	aContext isRenderingMethod 
		ifTrue: [ super checkMethod: aContext ]! !

!WAUnnecessaryWithSentToBrushRule methodsFor: 'accessing' stamp: 'lr 2/24/2009 15:07'!
group
	^ 'Seaside'! !

!WAUnnecessaryWithSentToBrushRule methodsFor: 'initialization' stamp: 'lr 2/28/2009 21:09'!
initialize
	super initialize.
	self matcher
		matches: '`html `msg with: ``@arg'
		do: [ :node :answer |
			(answer isNil
				and: [ (node parent isCascade not)
				and: [ (WASlime isBrushSelector: node receiver selector)
				and: [ (WASlime isBrushSelector: node receiver selector , ':')
				and: [ (WASlime isBrushSelector: node methodNode selector) not ] ] ] ])
					ifTrue: [ node ]
					ifFalse: [ answer ] ]! !

!WAUnnecessaryWithSentToBrushRule methodsFor: 'accessing' stamp: 'lr 2/27/2009 22:40'!
name
	^ 'Unnecessary #with: sent to brush'! !

!WAUnnecessaryWithSentToBrushRule methodsFor: 'accessing' stamp: 'lr 2/24/2009 15:07'!
rationale
	^ 'Sending #with: is only required if attributes are specified too.'! !

WASlimeParseTreeLintRule subclass: #WAUsesWrongRendererRule
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Slime'!

!WAUsesWrongRendererRule methodsFor: 'accessing' stamp: 'lr 2/24/2009 15:07'!
group
	^ 'Seaside'! !

!WAUsesWrongRendererRule methodsFor: 'initialization' stamp: 'lr 2/24/2009 15:11'!
initialize
	super initialize.
	self matcher 
		matchesAnyOf: #(
			" JQuery: "
			'``@obj ajax html: [ :`r | | `@temps | `@.statements ]'
			'``@obj load html: [ :`r | | `@temps | `@.statements ]'
			'``@obj ajax script: [ :`r | | `@temps | `@.statements ]'
			'``@obj load script: [ :`r | | `@temps | `@.statements ]'
			'``@obj ajax text: [ :`r | | `@temps | `@.statements ]'
			'``@obj load text: [ :`r | | `@temps | `@.statements ]'
			" Scriptaculous: "
			'``@obj updater callback: [ :`r | | `@temps | `@.statements ]'
			'``@obj evaluator callback: [ :`r | | `@temps | `@.statements ]'
			'``@obj periodical callback: [ :`r | | `@temps | `@.statements ]'
			'``@obj autocompleter callback: [ :`r | | `@temps | `@.statements ]'
			'``@obj inPlaceEditor callback: [ :`r | | `@temps | `@.statements ]'
			'``@obj inPlaceCollectionEditor callback: [ :`r | | `@temps | `@.statements ]' )
		do: [ :node :answer | 
			(answer isNil and: [ node arguments first references: node receiver receiver name ])
				ifTrue: [ node arguments first ]
				ifFalse: [ answer ] ]! !

!WAUsesWrongRendererRule methodsFor: 'accessing' stamp: 'lr 2/24/2009 15:07'!
name
	^ 'Uses the wrong renderer'! !

!WAUsesWrongRendererRule methodsFor: 'accessing' stamp: 'lr 2/24/2009 15:07'!
rationale
	^ 'Within AJAX callbacks you need to use the renderer that is passed into the block, the renderer from the outer context is only valid during the rendering of the outer context.'! !

WASlimeParseTreeLintRule subclass: #WAWithHasToBeLastMessageInCascadeRule
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Slime'!

!WAWithHasToBeLastMessageInCascadeRule methodsFor: 'running' stamp: 'lr 2/27/2009 22:13'!
checkMethod: aContext
	aContext isRenderingMethod 
		ifTrue: [ super checkMethod: aContext ]! !

!WAWithHasToBeLastMessageInCascadeRule methodsFor: 'accessing' stamp: 'lr 2/24/2009 15:07'!
group
	^ 'Seaside'! !

!WAWithHasToBeLastMessageInCascadeRule methodsFor: 'initialization' stamp: 'lr 2/28/2009 21:09'!
initialize
	super initialize.
	self matcher
		matches: '`html `msg with: ``@arg'
		do: [ :node :answer | 
			(answer isNil 
				and: [ (node parent isCascade)
				and: [ (node parent messages last ~= node) 
				and: [ (WASlime isBrushSelector: node receiver selector) ] ] ]) 
					ifTrue: [ node ]
					ifFalse: [ answer ] ]! !

!WAWithHasToBeLastMessageInCascadeRule methodsFor: 'accessing' stamp: 'lr 2/24/2009 15:07'!
name
	^ '#with: has to be last message in cascade'! !

!WAWithHasToBeLastMessageInCascadeRule methodsFor: 'accessing' stamp: 'lr 2/24/2009 15:07'!
rationale
	^ 'Sending #with: triggers serialization of the brush attributes, any attribute being specified afterwards has no effect.'! !

Object subclass: #WASlime
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Slime'!

WASlime class
	instanceVariableNames: 'brushSelectors callbackSelectors attributeSelectors'!

WASlime class
	instanceVariableNames: 'brushSelectors callbackSelectors attributeSelectors'!

!WASlime class methodsFor: 'accessing' stamp: 'lr 2/28/2009 23:09'!
attributeSelectors
	^ attributeSelectors ifNil: [ attributeSelectors := self findAttributeSelectors ]! !

!WASlime class methodsFor: 'accessing' stamp: 'lr 2/28/2009 23:09'!
brushSelectors
	^ brushSelectors ifNil: [ brushSelectors := self findBrushSelectors ]! !

!WASlime class methodsFor: 'accessing' stamp: 'lr 2/28/2009 23:09'!
callbackSelectors
	^ callbackSelectors ifNil: [ callbackSelectors := self findCallbackSelectors ]! !

!WASlime class methodsFor: 'testing' stamp: 'jf 3/16/2009 08:55'!
class: aClass includesBehaviorNamed: aSymbol
	"In order to avoid static dependencies to classes, we do this check by name."
	^ (Smalltalk includesKey: aSymbol) and: [
		aClass includesBehavior: (Smalltalk at: aSymbol)]! !

!WASlime class methodsFor: 'private' stamp: 'lr 11/2/2009 20:09'!
findAttributeSelectors
	| matcher result |
	matcher := RBParseTreeSearcher new.
	matcher
		matchesAnyOf: #(
			'self addClass: `@value'
			'self addStyle: `@value'
			'self addMedia: `@value'
			'self attributeAt: `@key put: `@value'
			'self attributeAt: `@key ifAbsentPut: `@block'
			'self attributes at: `@key put: `@value'
			'self attributes at: `@key ifAbsentPut: `@block'
			'self attributes at: `@key append: `@value'
			'self attributes at: `@key append: `@value separator: `@separator' )
		do: [ :context :node | true ].
	result := IdentitySet new.
	(self findClasses: #WABrush) , (self findClasses: #WAHtmlElement) do: [ :class | 
		class methodDictionary keysAndValuesDo: [ :selector :method | 
			(matcher executeTree: method parseTree initialAnswer: false) 
				ifTrue: [ result add: selector ] ] ].
	#(callback: enabled: labels:)
		do: [ :each | result add: each ].
	^ result! !

!WASlime class methodsFor: 'private' stamp: 'lr 11/2/2009 20:09'!
findBrushSelectors
	| matcher result |
	matcher := RBParseTreeSearcher new.
	matcher
		matchesAnyMethodOf: #(
			'`selector ^ self tag: `#tag'
			'`selector ^ self brush: `@obj'
			'`selector: `aBlock self `selector with: `aBlock'
			'`selector ^ `class on: self' )
		do: [ :context :node | true ].
	result := IdentitySet new.
	(self findClasses: #WACanvas) do: [ :class | 
		class methodDictionary keysAndValuesDo: [ :selector :method | 
			(matcher executeTree: method parseTree initialAnswer: false) 
				ifTrue: [ result add: selector ] ] ].
	^ result! !

!WASlime class methodsFor: 'private' stamp: 'lr 12/23/2009 12:52'!
findCallbackSelectors
	^ #(callback: callback:json: callback:value: callback:passengers: callbackAccordion: callbackDraggable: callbackDroppable: callbackResizable: callbackSlider: callbackSortable: callbackTabs: defaultAction: html: respond: script: text: triggerArgument:callback: triggerAutocompleter: triggerInPlaceEditor: triggerPassenger: triggerSliderCallback: triggerSortable:callback: triggerTree:callback:) asIdentitySet! !

!WASlime class methodsFor: 'private' stamp: 'lr 2/28/2009 23:02'!
findClasses: aSymbol
	| class |
	class := Smalltalk classNamed: aSymbol.
	^ class isNil 
		ifTrue: [ #() ]
		ifFalse: [ class withAllSubclasses asArray ]
	! !

!WASlime class methodsFor: 'initialization' stamp: 'jf 9/30/2009 00:19'!
initialize
	GRPlatform current addToStartUpList: self! !

!WASlime class methodsFor: 'testing' stamp: 'lr 2/28/2009 23:13'!
isBrushSelector: aString
	Symbol 
		hasInterned: aString
		ifTrue: [ :symbol | ^ self brushSelectors includes: symbol ].
	^ false! !

!WASlime class methodsFor: 'testing' stamp: 'lr 2/28/2009 23:13'!
isCallbackSelector: aString 
	Symbol 
		hasInterned: aString
		ifTrue: [ :symbol | ^ self callbackSelectors includes: symbol ].
	^ false! !

!WASlime class methodsFor: 'testing' stamp: 'lr 2/28/2009 23:13'!
isWithinCallback: aNode
	| parent |
	parent := aNode parent 
		ifNil: [ ^ false ].
	(aNode isBlock 
		and: [ parent isMessage 
		and: [ self callbackSelectors includes: parent selector ] ])
			ifTrue: [ ^ true ].
	^ self isWithinCallback: parent! !

!WASlime class methodsFor: 'initialization' stamp: 'lr 2/28/2009 23:10'!
startUp
	brushSelectors := callbackSelectors := attributeSelectors := nil! !

!ParseTreeEnvironment methodsFor: '*seaside-slime' stamp: 'lr 11/2/2009 20:20'!
matcher
	^ matcher ifNil: [ matcher := RBParseTreeSearcher new ]! !

!GRPackage class methodsFor: '*seaside-slime' stamp: 'jf 9/30/2009 01:16'!
seasideSlime
	^ self new
		name: 'Seaside-Slime';
		description: 'Code critis for Seaside. Detects common types of bugs.';
		addDependency: 'Grease-Core';
		yourself! !

RBBlockLintRule subclass: #WASlimeBlockLintRule
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Slime'!

WASlimeBlockLintRule subclass: #WABasicNewInitializeMissingRule
	instanceVariableNames: 'matcher'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Slime'!

!WABasicNewInitializeMissingRule methodsFor: 'running' stamp: 'jf 12/16/2009 13:59'!
checkClass: aContext 
	(aContext selectedClass isMeta not and: 
		[ (aContext selectedClass includesSelector: #initialize) and: 
			[ (aContext selectedClass includesBehavior: GRObject) not and: 
				[ (aContext selectedClass includesBehavior: GRNotification) not and: 
					[ (aContext selectedClass includesBehavior: GRError) not and: 
						[ aContext selectedClass class methodDictionary noneSatisfy: 
							[ :each | 
							matcher 
								executeTree: each parseTree
								initialAnswer: false ] ] ] ] ] ]) ifTrue: 
		[ result 
			addClass: aContext selectedClass
			selector: #initialize ]! !

!WABasicNewInitializeMissingRule methodsFor: 'accessing' stamp: 'lr 2/24/2009 15:35'!
group
	^ 'Seaside Portability'! !

!WABasicNewInitializeMissingRule methodsFor: 'initialization' stamp: 'lr 11/2/2009 20:09'!
initialize
	super initialize.
	matcher := RBParseTreeSearcher new.
	matcher 
		matches: 'self basicNew initialize'
		do: [ :node :answer | true ]! !

!WABasicNewInitializeMissingRule methodsFor: 'accessing' stamp: 'lr 2/24/2009 15:35'!
name
	^ '#basicNew initialize is missing'! !

!WABasicNewInitializeMissingRule methodsFor: 'accessing' stamp: 'lr 2/24/2009 15:35'!
rationale
	^ '#initialize is not called implicitely when sending #new to an object in other Smalltalk dialects.'! !

WASlimeBlockLintRule subclass: #WACallSuperImplementationRule
	instanceVariableNames: 'classesAndSelectors superSelectors'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Slime'!

!WACallSuperImplementationRule methodsFor: 'running' stamp: 'lr 11/8/2009 17:52'!
checkClass: aContext
	| current |
	aContext selectedClass isMeta
		ifTrue: [ ^ self ].
	current := aContext selectedClass superclass.
	superSelectors := Set new.
	[ current isNil ] whileFalse: [
		classesAndSelectors
			at: current name
			ifPresent: [ :values | superSelectors addAll: values ].
		current := current superclass ]! !

!WACallSuperImplementationRule methodsFor: 'running' stamp: 'lr 11/8/2009 17:53'!
checkMethod: aContext
	aContext selectedClass isMeta
		ifTrue: [ ^ self ].
	((superSelectors includes: aContext selector)
		and: [ (aContext superMessages includes: aContext selector) not ])
			ifTrue: [ result addClass: aContext selectedClass selector: aContext selector ]! !

!WACallSuperImplementationRule methodsFor: 'accessing' stamp: 'lr 2/28/2009 22:07'!
group
	^ 'Seaside'! !

!WACallSuperImplementationRule methodsFor: 'initialization' stamp: 'lr 12/3/2009 10:58'!
initialize
	super initialize.
	classesAndSelectors := Dictionary new.
	classesAndSelectors
		at: #WAObject put: #( initialize postCopy );
		at: #WAPainter put: #( updateRoot: updateUrl: );
		at: #WAPresenter put: #( updateStates: initialRequest: );
		at: #WABrush put: #( setParent:canvas: with: );
		at: #WATagBrush put: #( before after openTag closeTag );
		at: #WARequestFilter put: #( handleFiltered: updateStates: );
		at: #WASession put: #( updateRoot: updateUrl: updateStates: )! !

!WACallSuperImplementationRule methodsFor: 'accessing' stamp: 'lr 2/28/2009 22:07'!
name
	^ 'Call super implementation'! !

!WACallSuperImplementationRule methodsFor: 'accessing' stamp: 'lr 2/28/2009 22:09'!
rationale
	^ 'Some template methods provided by the framework require to call super to work correctly.'! !

WASlimeBlockLintRule subclass: #WAChangesStateWhileRenderingRule
	instanceVariableNames: 'matcher'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Slime'!

!WAChangesStateWhileRenderingRule methodsFor: 'running' stamp: 'lr 2/24/2009 16:09'!
checkMethod: aContext 
	| matches vars |
	aContext isRenderingMethod 
		ifFalse: [ ^ self ].
	matches := matcher 
		executeTree: aContext parseTree
		initialAnswer: OrderedCollection new.
	matches isEmpty
		ifTrue: [ ^ self ].
	vars := aContext instVarNames.
	matches do: [ :each | 
		(vars includes: each name) ifTrue: [ 
			result 
				addClass: aContext selectedClass
				selector: aContext selector.
			result matcher 
				matches: each name , ' := ``@obj'
				do: [ :node :answer | 
					answer isNil 
						ifTrue: [ node ]
						ifFalse: [ answer ] ] ] ]! !

!WAChangesStateWhileRenderingRule methodsFor: 'accessing' stamp: 'lr 2/24/2009 15:35'!
group
	^ 'Seaside'! !

!WAChangesStateWhileRenderingRule methodsFor: 'initialization' stamp: 'lr 11/2/2009 20:09'!
initialize
	super initialize.
	matcher := RBParseTreeSearcher new.
	matcher 
		matches: '`var'
		do: [ :node :answer | 
			(node isWrite and: [ (WASlime isWithinCallback: node) not ])
				ifTrue: [ answer add: node ].
			answer ]! !

!WAChangesStateWhileRenderingRule methodsFor: 'accessing' stamp: 'lr 2/24/2009 15:35'!
name
	^ 'Changes state while rendering'! !

!WAChangesStateWhileRenderingRule methodsFor: 'accessing' stamp: 'lr 2/24/2009 15:35'!
rationale
	^ 'Application state should not be changed in the rendering code, use a callback to define state.'! !

!WAChangesStateWhileRenderingRule methodsFor: 'accessing' stamp: 'lr 2/24/2009 15:35'!
resultClass
	^ ParseTreeEnvironment! !

WASlimeBlockLintRule subclass: #WADeprecatedApiProtocolRule
	instanceVariableNames: 'matcher'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Slime'!

!WADeprecatedApiProtocolRule methodsFor: 'running' stamp: 'lr 2/27/2009 21:40'!
checkMethod: aContext
	| matches |
	matches := matcher
		executeTree: aContext parseTree
		initialAnswer: OrderedCollection new.
	matches do: [ :node |
		" This ensures that the description is of the form ClassName>>#selector:"
		(node arguments first isLiteral and: [ node arguments first value isString and: [ aContext selectedClass name asString , '>>#' , aContext selector asString ~= node arguments first value ] ]) ifTrue: [
			result addClass: aContext selectedClass selector: aContext selector.
			result addSearchString: node arguments first value ].
		" This ensures that the description is a string and ends with a dot."
		(node arguments size > 1 and: [ node arguments second isLiteral and: [ node arguments second value isString and: [ (node arguments second value endsWith: '.') not ] ] ]) ifTrue: [
			result addClass: aContext selectedClass selector: aContext selector.
			result addSearchString: node arguments second value ] ]! !

!WADeprecatedApiProtocolRule methodsFor: 'accessing' stamp: 'lr 2/24/2009 15:35'!
group
	^ 'Seaside'! !

!WADeprecatedApiProtocolRule methodsFor: 'initialization' stamp: 'lr 11/2/2009 20:09'!
initialize
	super initialize.
	matcher := RBParseTreeSearcher new.
	matcher 
		matchesAnyOf: #(
			'``@receiver greaseDeprecatedApi: ``@arg1'
			'``@receiver greaseDeprecatedApi: ``@arg1 details: ``@arg2' )
		do: [ :node :answer | answer add: node; yourself ]! !

!WADeprecatedApiProtocolRule methodsFor: 'accessing' stamp: 'lr 2/24/2009 15:35'!
name
	^ 'Invalid use of deprecated API protocol.'! !

!WADeprecatedApiProtocolRule methodsFor: 'accessing' stamp: 'lr 2/24/2009 15:35'!
rationale
	^ 'Make sure to follow the following pattern: self seasideDeprecatedApi: ''Class>>#selector'' details: ''Some description.'''! !

WASlimeBlockLintRule subclass: #WADirectlySubclassObjectRule
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Slime'!

!WADirectlySubclassObjectRule methodsFor: 'running' stamp: 'lr 11/8/2009 17:54'!
checkClass: aContext 
	| selectedClass |
	selectedClass := aContext selectedClass.
	(selectedClass superclass = Object and: [ selectedClass ~= GRObject ])
		ifTrue: [ result addClass: selectedClass ]! !

!WADirectlySubclassObjectRule methodsFor: 'accessing' stamp: 'lr 2/24/2009 15:35'!
group
	^ 'Seaside Portability'! !

!WADirectlySubclassObjectRule methodsFor: 'accessing' stamp: 'lr 2/24/2009 15:35'!
name
	^ 'Directly subclasses Object'! !

!WADirectlySubclassObjectRule methodsFor: 'accessing' stamp: 'jf 3/13/2009 16:16'!
rationale
	^ 'Smalltalk dialects initialize objects differently, consider subclassing from WAPlatformObject to get a consistent initialization behavior.'! !

!WADirectlySubclassObjectRule methodsFor: 'accessing' stamp: 'lr 2/24/2009 15:35'!
resultClass
	^ ClassEnvironment! !

WASlimeBlockLintRule subclass: #WADoNotUseWhileRenderingRule
	instanceVariableNames: 'matcher'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Slime'!

!WADoNotUseWhileRenderingRule methodsFor: 'running' stamp: 'lr 2/24/2009 16:15'!
checkMethod: aContext 
	| matches |
	aContext isRenderingMethod 	
		ifFalse: [ ^ self ].
	matches := matcher 
		executeTree: aContext parseTree
		initialAnswer: OrderedCollection new.
	matches do: [ :each | 
		result 
			addClass: aContext selectedClass
			selector: aContext selector.
		result addSearchString: each selector ]! !

!WADoNotUseWhileRenderingRule methodsFor: 'accessing' stamp: 'lr 2/24/2009 15:35'!
group
	^ 'Seaside'! !

!WADoNotUseWhileRenderingRule methodsFor: 'initialization' stamp: 'lr 11/2/2009 20:09'!
initialize
	super initialize.
	matcher := RBParseTreeSearcher new.
	matcher 
		matchesAnyOf: (self selectors
			collect: [ :each | 'self' , (self genericPatternForSelector: each) ])
		do: [ :node :answer |
			(WASlime isWithinCallback: node)
				ifFalse: [ answer add: node ].
			answer ].! !

!WADoNotUseWhileRenderingRule methodsFor: 'accessing' stamp: 'lr 2/24/2009 15:35'!
name
	^ 'Do not use while rendering'! !

!WADoNotUseWhileRenderingRule methodsFor: 'accessing' stamp: 'lr 2/24/2009 15:35'!
rationale
	^ 'Certain functionaly should not be used while rendering. For example: #call: and #answer: should only be used from within callback code.'! !

!WADoNotUseWhileRenderingRule methodsFor: 'private' stamp: 'lr 2/27/2009 21:41'!
selectors
	^ #( answer answer: call: show: show:onAnswer: show:onAnswer:delegation: lightbox: chooseFrom: chooseFrom:caption: confirm: inform: request: request:default: request:label: request:label:default: wait: addDecoration: addMessage: answer answer: authenticateWith:during: call: chooseFrom: chooseFrom:caption: chooseFrom:caption:onAnswer: chooseFrom:onAnswer: confirm: confirm:onAnswer: decorateWith:during: filterWith:during: home inform: inform:onAnswer: isolate: lightbox: removeDecoration: request: request:default: request:default:onAnswer: request:label: request:label:default: request:label:default:onAnswer: request:label:onAnswer: request:onAnswer: show: show:onAnswer: show:onAnswer:delegation: validateWith: wait: )! !

WASlimeBlockLintRule subclass: #WADoNotUseWithinCallbackRule
	instanceVariableNames: 'matcher'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Slime'!

!WADoNotUseWithinCallbackRule methodsFor: 'running' stamp: 'lr 2/24/2009 16:16'!
checkMethod: aContext
	| matches |
	aContext isRenderingMethod 	
		ifFalse: [ ^ self ].
	matches := matcher 
		executeTree: aContext parseTree
		initialAnswer: OrderedCollection new.
	matches do: [ :each | 
		result 
			addClass: aContext selectedClass
			selector: aContext selector.
		result addSearchString: each selector ]! !

!WADoNotUseWithinCallbackRule methodsFor: 'accessing' stamp: 'lr 2/24/2009 15:35'!
group
	^ 'Seaside'! !

!WADoNotUseWithinCallbackRule methodsFor: 'initialization' stamp: 'lr 11/2/2009 20:09'!
initialize
	super initialize.
	matcher := RBParseTreeSearcher new.
	matcher 
		matchesAnyOf: (self selectors
			collect: [ :each | '``@receiver' , (self genericPatternForSelector: each) ])
		do: [ :node :answer | 
			(WASlime isWithinCallback: node)
				ifTrue: [ answer add: node ].
			answer ]! !

!WADoNotUseWithinCallbackRule methodsFor: 'accessing' stamp: 'lr 2/24/2009 15:35'!
name
	^ 'Do not use within callback'! !

!WADoNotUseWithinCallbackRule methodsFor: 'accessing' stamp: 'lr 2/24/2009 15:35'!
rationale
	^ 'Certain functionaly should not be used within callbacks. For example: #addLoadScript: should only be used while rendering.'! !

!WADoNotUseWithinCallbackRule methodsFor: 'private' stamp: 'lr 2/27/2009 21:44'!
selectors
	^ #( addLoadScript: )! !

WASlimeBlockLintRule subclass: #WAEmptyStatementsRule
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Slime'!

!WAEmptyStatementsRule methodsFor: 'running' stamp: 'TestRunner 11/2/2009 21:12'!
checkMethod: aContext 
	| periods position |
	aContext parseTree nodesDo: [ :node |
		node isSequence ifTrue: [
			periods := OrderedCollection withAll: node periods.
			1 to: node statements size - 1 do: [ :index |
				position := periods removeFirst.
				((node statements at: index) stop < position and: [ position < (node statements at: index + 1) start ])
					ifFalse: [ ^ result addClass: aContext selectedClass selector: aContext selector ] ].
			(periods isEmpty or: [ periods size = 1 and: [ node statements notEmpty and: [ node statements last stop < periods first ] ] ])
				ifFalse: [ ^ result addClass: aContext selectedClass selector: aContext selector ] ] ]! !

!WAEmptyStatementsRule methodsFor: 'accessing' stamp: 'lr 9/8/2009 13:57'!
group
	^ 'Seaside Portability'! !

!WAEmptyStatementsRule methodsFor: 'accessing' stamp: 'lr 9/8/2009 13:59'!
name
	^ 'Empty Statements'! !

!WAEmptyStatementsRule methodsFor: 'accessing' stamp: 'lr 9/8/2009 14:12'!
rationale
	^ 'Empty statements are not supported by most Smalltalk implementations.'! !

!WAEmptyStatementsRule methodsFor: 'running' stamp: 'lr 9/8/2009 19:15'!
resetResult
	super resetResult.
	result addSearchString: '..'; addSearchString: '.'! !

WASlimeBlockLintRule subclass: #WAInstantiatesComponentWhileRenderingRule
	instanceVariableNames: 'matcher'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Slime'!

!WAInstantiatesComponentWhileRenderingRule methodsFor: 'running' stamp: 'lr 2/24/2009 16:17'!
checkMethod: aContext 
	| matches |
	aContext isRenderingMethod 
		ifFalse: [ ^ self ].
	matches := matcher 
		executeTree: aContext parseTree 
		initialAnswer: OrderedCollection new.
	matches do: [ :each | 
		result 
			addClass: aContext selectedClass
			selector: aContext selector.
		result addSearchString: each receiver name ]! !

!WAInstantiatesComponentWhileRenderingRule methodsFor: 'accessing' stamp: 'lr 2/24/2009 15:35'!
group
	^ 'Seaside'! !

!WAInstantiatesComponentWhileRenderingRule methodsFor: 'initialization' stamp: 'lr 11/2/2009 20:09'!
initialize
	super initialize.
	matcher := RBParseTreeSearcher new.
	matcher 
		matches: '`receiver `@message: ``@args'
		do: [ :node :answer |
			| class |
			((class := Smalltalk classNamed: node receiver name) notNil and: [ class isBehavior and: [ (WASlime class: class includesBehaviorNamed: #WAPresenter) and: [ (WASlime isWithinCallback: node) not ] ] ])
				ifTrue: [ answer add: node ].
			answer ]! !

!WAInstantiatesComponentWhileRenderingRule methodsFor: 'accessing' stamp: 'lr 2/24/2009 15:35'!
name
	^ 'Instantiates component while rendering'! !

!WAInstantiatesComponentWhileRenderingRule methodsFor: 'accessing' stamp: 'lr 2/24/2009 15:35'!
rationale
	^ 'Components should only be instanciated in initialization-code, callbacks or through lazy initialization.'! !

WASlimeBlockLintRule subclass: #WAInvalidObjectInitializationRule
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Slime'!

!WAInvalidObjectInitializationRule methodsFor: 'running' stamp: 'lr 11/8/2009 17:55'!
checkMethod: aContext 
	(aContext selectedClass isMeta not 
		and: [ (aContext selector beginsWith: #initialize) 
		and: [ aContext selector numArgs > 0
		and: [ aContext selectedClass ~= GRObject
		and: [ (aContext selectedClass includesBehavior: GRObject)
		and: [ (aContext superMessages includes: aContext selector) not 
		and: [ aContext selfMessages noneSatisfy: [ :each | each beginsWith: #initialize ] ] ] ] ] ] ]) 
			ifTrue: [ result addClass: aContext selectedClass selector: aContext selector ]! !

!WAInvalidObjectInitializationRule methodsFor: 'accessing' stamp: 'lr 2/24/2009 15:35'!
group
	^ 'Seaside Portability'! !

!WAInvalidObjectInitializationRule methodsFor: 'accessing' stamp: 'lr 2/24/2009 15:35'!
name
	^ 'Invalid object initialization'! !

!WAInvalidObjectInitializationRule methodsFor: 'accessing' stamp: 'lr 2/24/2009 15:35'!
rationale
	^ 'Initialize methods are required to call the designated initializer or its super implementation.'! !

!WASlimeBlockLintRule class methodsFor: 'testing' stamp: 'lr 2/24/2009 17:10'!
isVisible
	^ self name ~= #WASlimeBlockLintRule! !

WASlimeBlockLintRule subclass: #WAUsesCurlyBraceArraysRule
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Slime'!

!WAUsesCurlyBraceArraysRule methodsFor: 'running' stamp: 'lr 2/27/2009 21:47'!
checkMethod: aContext 
	aContext compiledMethod literals do: [ :each | 
		(each isSymbol and: [ self selectors includes: each ]) ifTrue: [
			result 
				addClass: aContext selectedClass
				selector: aContext selector.
			result addSearchString: '{' ] ]! !

!WAUsesCurlyBraceArraysRule methodsFor: 'accessing' stamp: 'lr 2/24/2009 15:35'!
group
	^ 'Seaside Portability'! !

!WAUsesCurlyBraceArraysRule methodsFor: 'accessing' stamp: 'lr 2/24/2009 15:35'!
name
	^ 'Uses curly brace arrays'! !

!WAUsesCurlyBraceArraysRule methodsFor: 'accessing' stamp: 'lr 2/24/2009 15:35'!
rationale
	^ 'Curly brace expressions are not portable accross different Smalltalk dialects.'! !

!WAUsesCurlyBraceArraysRule methodsFor: 'private' stamp: 'lr 2/27/2009 21:47'!
selectors
	^ #( braceStream: braceWith: braceWith:with: braceWith:with:with: braceWith:with:with:with: braceWithNone )! !

WASlimeBlockLintRule subclass: #WAUsesLiteralByteArraysRule
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Slime'!

!WAUsesLiteralByteArraysRule methodsFor: 'running' stamp: 'lr 2/24/2009 16:21'!
checkMethod: aContext 
	aContext compiledMethod literals do: [ :each | 
		(each isLiteral and: [ each isKindOf: ByteArray ]) ifTrue: [
			result 
				addClass: aContext selectedClass
				selector: aContext selector.
			result addSearchString: '#[' ] ]! !

!WAUsesLiteralByteArraysRule methodsFor: 'accessing' stamp: 'lr 2/24/2009 15:35'!
group
	^ 'Seaside Portability'! !

!WAUsesLiteralByteArraysRule methodsFor: 'accessing' stamp: 'lr 2/24/2009 15:35'!
name
	^ 'Uses literal byte arrays'! !

!WAUsesLiteralByteArraysRule methodsFor: 'accessing' stamp: 'lr 2/24/2009 15:35'!
rationale
	^ 'Literal byte arrays are not portable accross different Smalltalk dialects.'! !

WASlimeBlockLintRule subclass: #WAUsesMethodAnnotationsRule
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Slime'!

!WAUsesMethodAnnotationsRule methodsFor: 'running' stamp: 'lr 2/27/2009 21:51'!
checkMethod: aContext 
	aContext compiledMethod pragmas do: [ :each |
		result 
			addClass: aContext selectedClass
			selector: aContext selector.
		result addSearchString: each keyword ]! !

!WAUsesMethodAnnotationsRule methodsFor: 'accessing' stamp: 'lr 2/27/2009 21:49'!
group
	^ 'Seaside Portability'! !

!WAUsesMethodAnnotationsRule methodsFor: 'accessing' stamp: 'lr 2/27/2009 21:49'!
name
	^ 'Uses method annotations'! !

!WAUsesMethodAnnotationsRule methodsFor: 'accessing' stamp: 'lr 2/27/2009 21:52'!
rationale
	^ 'As of now, it is unclear if method-annotations/pragmas are compatible across all supported platforms. Likely this rule will soon be removed.'! !

WASlimeBlockLintRule subclass: #WAUsesNotPortableClassRule
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Slime'!

!WAUsesNotPortableClassRule methodsFor: 'running' stamp: 'lr 2/27/2009 22:14'!
checkClass: aContext 
	aContext selectedClass allSuperclassesDo: [ :each |
		(self classNames includes: each name) ifTrue: [
			result addClass: aContext selectedClass.
			result addSearchString: each name ] ]! !

!WAUsesNotPortableClassRule methodsFor: 'running' stamp: 'lr 2/27/2009 22:14'!
checkMethod: aContext 
	aContext compiledMethod literals do: [ :each | 
		(each isVariableBinding and: [ self classNames includes: each key ]) ifTrue: [
			result 
				addClass: aContext selectedClass
				selector: aContext selector.
			result addSearchString: each key ] ]! !

!WAUsesNotPortableClassRule methodsFor: 'private' stamp: 'jf 12/18/2009 14:16'!
classNames
	^ #( Delay MIMEDocument Monitor Mutex MutexSet Random Semaphore TimeStamp Timestamp MessageSend )! !

!WAUsesNotPortableClassRule methodsFor: 'accessing' stamp: 'lr 2/24/2009 15:35'!
group
	^ 'Seaside Portability'! !

!WAUsesNotPortableClassRule methodsFor: 'accessing' stamp: 'lr 2/24/2009 15:35'!
name
	^ 'Uses not portable class'! !

!WAUsesNotPortableClassRule methodsFor: 'accessing' stamp: 'lr 2/24/2009 15:35'!
rationale
	^ 'Some classes are not portable accross different Smalltalk dialects.'! !

!SmalllintContext methodsFor: '*seaside-slime' stamp: 'lr 1/11/2009 23:43'!
isRenderingMethod
	(self selector numArgs > 0 and: [ self selector beginsWith: #render ])
		ifFalse: [ ^ false ].
	self compiledMethod literals do: [ :each |
		(each isSymbol and: [ WASlime isBrushSelector: each ])
			ifTrue: [ ^ true ] ].
	^ false! !

RBTransformationRule subclass: #WASlimeTransformationRule
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Slime'!

WASlimeTransformationRule subclass: #WAAnsiBooleansRule
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Slime'!

!WAAnsiBooleansRule methodsFor: 'accessing' stamp: 'lr 2/24/2009 15:37'!
group
	^ 'ANSI Compatibility'! !

!WAAnsiBooleansRule methodsFor: 'initialization' stamp: 'lr 2/24/2009 15:27'!
initialize
	super initialize.
	self rewriteRule
		replace: '``@expr1 and: [ | `@temp2 | ``@expr2 ] and: [ | `@temp3 | ``@expr3 ]'
			with: '``@expr1 and: [ | `@temp2 | ``@expr2 and: [ | `@temp3 | ``@expr3 ] ]';
		replace: '``@expr1 and: [ | `@temp2 | ``@expr2 ] and: [ | `@temp3 | ``@expr3 ] and: [ | `@temp4 | ``@expr4 ]'
			with: '``@expr1 and: [ | `@temp2 | ``@expr2 and: [ | `@temp3 | ``@expr3 and: [ | `@temp4 | ``@expr4 ] ] ]';
		replace: '``@expr1 and: [ | `@temp2 | ``@expr2 ] and: [ | `@temp3 | ``@expr3 ] and: [ | `@temp4 | ``@expr4 ] and: [  | `@temp5 | ``@expr5 ]'
			with: '``@expr1 and: [ | `@temp2 | ``@expr2 and: [ | `@temp3 | ``@expr3 and: [ | `@temp4 | ``@expr4 and: [ | `@temp5 | ``@expr5 ] ] ] ]';
		replace: '``@expr1 or: [ | `@temp2 | ``@expr2 ] or: [ | `@temp3 | ``@expr3 ]'
			with: '``@expr1 or: [ | `@temp2 | ``@expr2 or: [ | `@temp3 | ``@expr3 ] ]';
		replace: '``@expr1 or: [ | `@temp2 | ``@expr2 ] or: [ | `@temp3 | ``@expr3 ] or: [ | `@temp4 | ``@expr4 ]'
			with: '``@expr1 or: [ | `@temp2 | ``@expr2 or: [ | `@temp3 | ``@expr3 or: [ | `@temp4 | ``@expr4 ] ] ]';
		replace: '``@expr1 or: [ | `@temp2 | ``@expr2 ] or: [ | `@temp3 | ``@expr3 ] or: [ | `@temp4 | ``@expr4 ] or: [ | `@temp5 | ``@expr5 ]'
			with: '``@expr1 or: [ | `@temp2 | ``@expr2 or: [ | `@temp3 | ``@expr3 or: [ | `@temp4 | ``@expr4 or: [ | `@temp5 | ``@expr5 ] ] ] ]';
		replace: '``@expr1 or: [ | `@temp2 | ``@expr2 ] or: [ | `@temp3 | ``@expr3 ] or: [ | `@temp4 | ``@expr4 ] or: [ | `@temp5 | ``@expr5 ] or: [ | `@temp6 | ``@expr6 ]'
			with: '``@expr1 or: [ | `@temp2 | ``@expr2 or: [ | `@temp3 | ``@expr3 or: [ | `@temp4 | ``@expr4 or: [ | `@temp5 | ``@expr5 or: [ | `@temp6 | ``@expr6 ] ] ] ] ]'! !

!WAAnsiBooleansRule methodsFor: 'accessing' stamp: 'lr 2/24/2009 15:37'!
name
	^ 'Booleans'! !

WASlimeTransformationRule subclass: #WAAnsiCharactersRule
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Slime'!

!WAAnsiCharactersRule methodsFor: 'accessing' stamp: 'lr 7/16/2009 08:35'!
group
	^ 'ANSI Compatibility'! !

!WAAnsiCharactersRule methodsFor: 'initialization' stamp: 'jok 9/19/2009 14:25'!
initialize
	super initialize.
	self rewriteRule
		replace: 'Character value: ``@expr' with: 'Character codePoint: ``@expr';
		replace: 'String tab' with: '(String with: Character tab)';
		replace: 'String cr' with: '(String with: Character cr)';
		replace: 'String lf' with: '(String with: Character lf)';
		replace: 'String crlf' with: '(String with: Character cr with: Character lf)';
		replace: 'String space' with: '(String with: Character space)'! !

!WAAnsiCharactersRule methodsFor: 'accessing' stamp: 'lr 7/16/2009 08:35'!
name
	^ 'Characters'! !

WASlimeTransformationRule subclass: #WAAnsiCollectionsRule
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Slime'!

!WAAnsiCollectionsRule methodsFor: 'accessing' stamp: 'lr 2/24/2009 15:37'!
group
	^ 'ANSI Compatibility'! !

!WAAnsiCollectionsRule methodsFor: 'initialization' stamp: 'lr 2/28/2009 21:41'!
initialize
	super initialize.
	self rewriteRule
		replace: '``@collection withIndexDo: [ :`each :`index |  | `@temps | ``@.body ]'
			with: '``@collection keysAndValuesDo: [ :`index :`each |  | `@temps | ``@.body ]';
		replace: '``@collection doWithIndex: [ :`each :`index |  | `@temps | ``@.body ]'
			with: '``@collection keysAndValuesDo: [ :`index :`each |  | `@temps | ``@.body ]';
		replace: '``@collection collect: ``@block1 thenDo: ``@block2'
			with: '(``@collection collect: ``@block1) do: ``@block2';
		replace: '``@collection collect: ``@block1 thenSelect: ``@block2'
			with: '(``@collection collect: ``@block1) select: ``@block2';
		replace: '``@collection pairsDo: [ :`t1 :`t2 | ``@.statements ]'
			with: '1 to: ``@collection size by: 2 do:  [ :index | | `t1 `t2 | t1 := ``@collection at: index. t2 := ``@collection at: index + 1. ``@.statements ]';
		replace: '``@collection reject: ``@block1 thenDo: ``@block2'
			with: '(``@collection reject: ``@block1) do: ``@block2';
		replace: '``@collection select: ``@block1 thenCollect: ``@block2'
			with: '(``@collection select: ``@block1) collect: ``@block2';
		replace: '``@collection select: ``@block1 thenDo: ``@block2'
			with: '(``@collection select: ``@block1) do: ``@block2'! !

!WAAnsiCollectionsRule methodsFor: 'accessing' stamp: 'lr 2/24/2009 15:37'!
name
	^ 'Collections'! !

WASlimeTransformationRule subclass: #WAAnsiConditionalsRule
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Slime'!

!WAAnsiConditionalsRule methodsFor: 'accessing' stamp: 'lr 2/24/2009 15:36'!
group
	^ 'ANSI Compatibility'! !

!WAAnsiConditionalsRule methodsFor: 'initialization' stamp: 'lr 2/28/2009 21:38'!
initialize
	super initialize.
	self rewriteRule
		replace: '``@boolean ifNotNilDo: ``@block'
			with: '``@boolean ifNotNil: ``@block';
		replace: '``@boolean ifNotNilDo: ``@block1 ifNil: ``@block2'
			with: '``@boolean ifNotNil: ``@block1 ifNil: ``@block2';
		replace: '``@boolean ifNil: ``@block1 ifNotNilDo: ``@block2'
			with: '``@boolean ifNil: ``@block1 ifNotNil: ``@block2';
		replace: '``@boolean ifNotNil: [ | `@temps | ``@.body ]'
			with: '``@boolean ifNotNil: [ :arg | | `@temps | ``@.body ]';
		replace: '``@boolean ifNotNil: [ | `@temps | ``@.body ] ifNil: ``@block '
			with: '``@boolean ifNotNil: [ :arg | | `@temps | ``@.body ] ifNil: ``@block';
		replace: '``@boolean ifNil: ``@block ifNotNil: [ | `@temps | ``@.body ]'
			with: '``@boolean ifNil: ``@block ifNotNil: [ :arg | | `@temps | ``@.body ]'! !

!WAAnsiConditionalsRule methodsFor: 'accessing' stamp: 'lr 2/24/2009 15:37'!
name
	^ 'Conditionals'! !

WASlimeTransformationRule subclass: #WAAnsiConvertorRule
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Slime'!

!WAAnsiConvertorRule methodsFor: 'accessing' stamp: 'lr 4/14/2009 00:32'!
group
	^ 'ANSI Compatibility'! !

!WAAnsiConvertorRule methodsFor: 'initialization' stamp: 'jf 12/8/2009 23:17'!
initialize
	super initialize.
	self rewriteRule
		replace: '``@object asString' with: '``@object greaseString';
		replace: '``@object asInteger' with: '``@object greaseInteger'! !

!WAAnsiConvertorRule methodsFor: 'accessing' stamp: 'lr 4/14/2009 00:33'!
name
	^ 'Convertors'! !

WASlimeTransformationRule subclass: #WAAnsiExceptionsRule
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Slime'!

!WAAnsiExceptionsRule methodsFor: 'accessing' stamp: 'lr 2/24/2009 15:36'!
group
	^ 'ANSI Compatibility'! !

!WAAnsiExceptionsRule methodsFor: 'initialization' stamp: 'lr 2/28/2009 21:44'!
initialize
	super initialize.
	self rewriteRule
		replace: '``@block on: `class do: [ | `@temps | ``@.body ]'
			with: '``@block on: `class do: [ :err | | `@temps | ``@.body ]'! !

!WAAnsiExceptionsRule methodsFor: 'accessing' stamp: 'lr 2/24/2009 15:37'!
name
	^ 'Exceptions'! !

WASlimeTransformationRule subclass: #WAAnsiStreamsRule
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Slime'!

!WAAnsiStreamsRule methodsFor: 'accessing' stamp: 'lr 2/24/2009 15:36'!
group
	^ 'ANSI Compatibility'! !

!WAAnsiStreamsRule methodsFor: 'initialization' stamp: 'lr 2/24/2009 15:29'!
initialize
	super initialize.
	self rewriteRule
		replace: '`{ :node :context | node isVariable and: [ (Smalltalk includesKey: node name asSymbol) not and: [ context at: ''`receiver'' ifAbsentPut: [ node ]. true ] ] } cr'
			with: '`{ :context | context at: ''`receiver'' } nextPut: Character cr';
		replace: '`{ :node :context | node isVariable and: [ (Smalltalk includesKey: node name asSymbol) not and: [ context at: ''`receiver'' ifAbsentPut: [ node ]. true ] ] } lf'
			with: '`{ :context | context at: ''`receiver'' } nextPut: Character lf';
		replace: '``@stream nextPut: Character cr; nextPut: Character lf'
			with: '``@stream crlf';
		replace: '``@collection writeStream'
			with: 'WriteStream on: ``@collection'! !

!WAAnsiStreamsRule methodsFor: 'accessing' stamp: 'lr 2/24/2009 15:52'!
name
	^ 'Streams'! !

WASlimeTransformationRule subclass: #WAHandleRequestRule
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Slime'!

!WAHandleRequestRule methodsFor: 'running' stamp: 'jf 3/16/2009 09:04'!
checkMethod: aContext
	(WASlime class: aContext selectedClass includesBehaviorNamed: #WARequestHandler)
		ifTrue: [ super checkMethod: aContext ]! !

!WAHandleRequestRule methodsFor: 'accessing' stamp: 'lr 2/24/2009 15:37'!
group
	^ 'Seaside 2.9'! !

!WAHandleRequestRule methodsFor: 'initialization' stamp: 'lr 11/2/2009 20:09'!
initialize
	super initialize.
	self rewriteRule
		replaceMethod: 'handleRequest: `aRequest | `@temps | `@.statements'
		withValueFrom: [ :node |
			RBParseTreeRewriter new
				replace: node arguments first name with: 'aRequestContext';
				executeTree: node.
			RBParseTreeRewriter new
				replace: 'aRequestContext' with: 'aRequestContext request';
				executeTree: node body.
			node
				selector: #handleFiltered:;
				arguments: (Array with: (RBVariableNode named: 'aRequestContext'));
				yourself ]! !

!WAHandleRequestRule methodsFor: 'accessing' stamp: 'lr 2/24/2009 15:22'!
name
	^ 'Rename WARequestHandler>>#handleRequest: to #handleFiltered:'! !

WASlimeTransformationRule subclass: #WASessionExpiredRule
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Slime'!

!WASessionExpiredRule methodsFor: 'running' stamp: 'jf 3/16/2009 08:56'!
checkMethod: aContext
	(WASlime class: aContext selectedClass includesBehaviorNamed: #WASession)
		ifTrue: [ super checkMethod: aContext ]! !

!WASessionExpiredRule methodsFor: 'accessing' stamp: 'lr 2/24/2009 15:37'!
group
	^ 'Seaside 2.9'! !

!WASessionExpiredRule methodsFor: 'initialization' stamp: 'lr 2/24/2009 15:30'!
initialize
	super initialize.
	self rewriteRule
		replace: 'expired 
			| `@temps | 
			`@.Statements'
		with: 'unregistered
			| `@temps | 
			`@.Statements'! !

!WASessionExpiredRule methodsFor: 'accessing' stamp: 'lr 2/24/2009 15:22'!
name
	^ 'Rename WASession>>#expired to #unregistered'! !

!WASlimeTransformationRule class methodsFor: 'testing' stamp: 'lr 6/15/2009 10:32'!
isVisible
	^ self name ~= #WASlimeTransformationRule! !
WASlime initialize!
